#include "can.h"
#include "c_can.h"

void ccanInit(){

	int i;
	/** - Setup control register
	*     - Disable automatic wakeup on bus activity
	*     - Local power down mode disabled
	*     - Disable DMA request lines
	*     - Enable global Interrupt Line 0 and 1
	*     - Disable debug mode
	*     - Release from software reset
	*     - Disable parity and ECC
	*     - Enable/Disable auto bus on timer
	*     - Setup message completion before entering debug state
	*     - Setup normal operation mode
	*     - Request write access to the configuration registers
	*     - Setup automatic retransmission of messages
	*     - Disable error interrups
	*     - Disable status interrupts
	*     - Enter initialization mode
	*/
	canREG1->CTL = 0x00000000U | 0x00000000U | 0x00021443U;

	canREG1->INTMUXx[0U] = 0x00000000U; // no interrupts
	canREG1->INTMUXx[1U] = 0x00000000U;

	/** - Setup auto bus on timer pewriod */
	canREG1->ABOTR = 0U;

	// todo: check various masks and if things do or do not get accepted.  Xtd and Std.
	for(i=1; i<=16; i++){
		if(i<=8){
			can_msg_set(i, MSG_DIR_RECEIVE, MSG_ID_TYPE_STD, i, 0x00F);
		} else {
			can_msg_set(i, MSG_DIR_TRANSMIT, MSG_ID_TYPE_STD, i, 0x00F);
			// todo: does the transmit id matter?
		}
	}

	/** - Setup IF1 for data transmission
	*     - Wait until IF1 is ready for use
	*     - Set IF1 control byte
	*/
	while (CAN1_IF1_BUSY);

	canREG1->IF1CMD  = 0x87;

	/** - Setup IF2 for reading data
	*     - Wait until IF1 is ready for use
	*     - Set IF1 control byte
	*/
	while (CAN1_IF2_BUSY);

	canREG1->IF2CMD = 0x17;

	/** - Setup bit timing
	*     - Setup baud rate prescaler extension
	*     - Setup TSeg2
	*     - Setup TSeg1
	*     - Setup sample jump width
	*     - Setup baud rate prescaler
	*/
	canREG1->BTR = (0U << 16U) |
				   ((2U - 1U) << 12U) |
				   (((3U + 2U) - 1U) << 8U) |
				   ((2U - 1U) << 6U) |
				   19U;

	/** - Setup TX pin to functional output */
	canREG1->TIOC = 0x0000004CU;

	/** - Setup RX pin to functional input */
	canREG1->RIOC = 0x00000048U;

	/** - Leave configuration and initialization mode  */
	canREG1->CTL &= ~0x00000041U;

}


// TODO: setup FIFO properly!
// should we be passing direction?
void can_msg_set(int mailbox, enum MSG_DIR dir, enum MSG_ID_TYPE id_type, int id, int mask){
	/** - Initialize message
    *     - Wait until IF1 is ready for use
    *     - Set message mask
    *     - Set message control word
    *     - Set message arbitration
    *     - Set IF1 control byte
    *     - Set IF1 message number
    */
    while (CAN1_IF1_BUSY); // could be improved by using either IF1 or IF2 if busy

    if(id_type == MSG_ID_TYPE_XTD){
    	canREG1->IF1MSK.bit.MXtd = 1;
    	canREG1->IF1ARB.bit.Xtd = 1;
    	canREG1->IF1MSK.bit.Msk = mask;
    	canREG1->IF1ARB.bit.ID =  id;
    } else { // MSG_ID_TYPE_STD
    	canREG1->IF1MSK.bit.MXtd = 1; // this makes sure 0x1FFFFFFF isn't the same as 0x7FF
		canREG1->IF1ARB.bit.Xtd = 0;
		canREG1->IF1MSK.bit.Msk = mask << 18; // the mask and ID now need to be in the high portion
		canREG1->IF1ARB.bit.ID =  id << 18;
    }

    canREG1->IF1MSK.bit.MDir = 1;
    canREG1->IF1ARB.bit.MsgVal = 1;

    if(dir == MSG_DIR_TRANSMIT){
    	canREG1->IF1ARB.bit.Dir = MSG_DIR_TRANSMIT;
    } else {
    	canREG1->IF1ARB.bit.Dir = MSG_DIR_RECEIVE;
    }
    canREG1->IF1MCTL.bit.UMask = 1;
    canREG1->IF1MCTL.bit.EoB = 1;
    canREG1->IF1MCTL.bit.DLC = 8;
    canREG1->IF1CMD  = IF_CMD_WR | IF_UPDATE_MASK | IF_UPDATE_ARB |
    		IF_UPDATE_CONTROL | IF_UPDATE_CLR;  // specify which portions we are writing to
    canREG1->IF1NO   = mailbox; // setting the number starts the write from IF1 to Msg RAM
}

// For now assume size fixed to 16 with canREG1
#define NUMBER_OF_MAILBOXES 16

// Copies from message RAM to local RAM through interface registers
// assumes that mailbox directions don't change between calls.
// assumes that RAM is setup with receive boxes first then transmit
// update_action- type of update to do.  Receive, Transmit, Both.
void can_update_buffer(enum MSG_DIR update_action, struct CAN_MESSAGE *local_msgs){
	// Receive: scan through message RAM for any receive mailboxes, update local RAM if new
	//    	should we set a flag if we are overwriting an un-read mailbox?
	// Transmit: scan through local RAM for transmit mailboxes update message RAM
	//		should we set a flag if we are overwriting an un-sent mailbox?
	// Both: do receive then transmit

	int msg_mb_num = 0;
	enum CAN_MB_UPDATE_STATUS returnVal = SUCCESS;

	for(msg_mb_num = 0; msg_mb_num < NUMBER_OF_MAILBOXES; msg_mb_num++){
		// do receive update if desired and mailbox is receiving
		if((local_msgs[msg_mb_num].dir == MSG_DIR_RECEIVE) && (update_action == MSG_DIR_RECEIVE)){
			if(newRcvMsg(msg_mb_num)){
				returnVal = update_rx_mb_num(msg_mb_num, &local_msgs[msg_mb_num]); // TODO: do something with return
				while(returnVal);
			}
		}
		if((local_msgs[msg_mb_num].dir == MSG_DIR_TRANSMIT) && (update_action == MSG_DIR_TRANSMIT)){
			if(local_msgs[msg_mb_num].state == MSG_STATE_READY){
				returnVal = update_tx_mb_num(msg_mb_num, &local_msgs[msg_mb_num]); // TODO: do something with return
				while(returnVal);
			}
		}
// todo: what to return?
	} // todo: could speed up by assuming not each message needs to be checked trans/rcv.  Assume ordered in some way?
	return;
}

// returns true if the mailbox has a new message
BOOLEAN newRcvMsg(int num){
	return (BOOLEAN)((canREG1->NWDATx[0]) & (1 << (num)));
}

// TODO: do we need a mark of if it's new/read or should we just be queing it?
// does the local_RAM struct need a dir?
// should local_RAM be exact mailbox address or base?
enum CAN_MB_UPDATE_STATUS update_rx_mb_num(int num, struct CAN_MESSAGE *local_msg){



	num++;  // account for the fact that the message ram starts at zero

	canREG1->IF1CMD  = IF_UPDATE_ALL | IF_CMD_RD; // transfer everything.  Any reason not to?
	while(CAN1_IF1_BUSY); // wait until interface is not busy
	// TODO: which bits of the IF1CMD reg do we set.  Could it be faster if not all?
	canREG1->IF1NO = num; // do the copying
	while(CAN1_IF1_BUSY); // wait until interface is done copying

	// check if dir is proper for rcv
	if(canREG1->IF1ARB.bit.Dir == MSG_DIR_TRANSMIT){
		return NON_MATCHING_MB_DIR;
	}
	// check if MsgLst is set and then possibly reset
	if(canREG1->IF1MCTL.bit.MsgLst){
		// TODO: should we reset the bit in message RAM?
		return RECEIVE_OVERRUN;
	}
	if(!canREG1->IF1MCTL.bit.NewDat){
		return NO_NEW_CAN_RX_DATA_AVAILABLE;
	}

	// we are not updating dir because it is already set and is also check above
	// get dir, data, and ID
	// todo: what about the fact that the rcv filter on the mailbox can be reset by incoming data?
	if(canREG1->IF1ARB.bit.Xtd){
		local_msg->id = canREG1->IF1ARB.bit.ID;
	} else {
		local_msg->id = canREG1->IF1ARB.bit.ID >> 18;
	}
	local_msg->dlc = canREG1->IF1MCTL.bit.DLC;
	// since the data is not kept in an orderly arrangement we'll just do w/o a loop
	local_msg->data[0] = canREG1->IF1DATx[3];
	local_msg->data[1] = canREG1->IF1DATx[2];
	local_msg->data[2] = canREG1->IF1DATx[1];
	local_msg->data[3] = canREG1->IF1DATx[0];
	local_msg->data[4] = canREG1->IF1DATx[7];
	local_msg->data[5] = canREG1->IF1DATx[6];
	local_msg->data[6] = canREG1->IF1DATx[5];
	local_msg->data[7] = canREG1->IF1DATx[4];
	local_msg->state = MSG_STATE_READY;
	if(local_msg->dir == MSG_DIR_TRANSMIT){ // todo: where is dir updated
		return NON_MATCHING_MB_DIR;
	}

	// Todo: can we combine this with one above?
	// TODO: if the other portion of the MCTL regs are different among mailboxes this will corrupt their individual values.  Need to write just the one bit!
	canREG1->IF1MCTL.bit.NewDat = 0; // clear the new data marker since we just pulled the message
	canREG1->IF1CMD = IF_CMD_WR | IF_UPDATE_CONTROL;
	while(CAN1_IF1_BUSY); // wait until interface is not busy
	canREG1->IF1NO = num; // do the copying
	while(CAN1_IF1_BUSY); // wait until interface is done copying

	return SUCCESS;
}


// does it really make since to write the ID and DLC?
enum CAN_MB_UPDATE_STATUS update_tx_mb_num(int num, struct CAN_MESSAGE *local_msg){



	num++;  // account for the fact that the message ram starts at zero

	// check if dir is proper for transmit
	if(local_msg->dir == MSG_DIR_RECEIVE){
		return NON_MATCHING_MB_DIR;
	}
	// does it make since to check the message ram direction bit


	while(CAN1_IF1_BUSY); // wait until interface is not busy
	canREG1->IF1CMD = IF_CMD_RD | IF_UPDATE_CONTROL; // just get the status bits
	canREG1->IF1NO = num; // do the copying
	while(CAN1_IF1_BUSY); // wait until interface is done copying

	// check if new data still in ram
	if(local_msg->state == MSG_STATE_EMPTY){
		return NO_NEW_CAN_TX_DATA_READY;
	}
	if(canREG1->IF1MCTL.bit.TxRqst){
		return TRANSMIT_OVERRUN;
	}


	canREG1->IF1ARB.bit.ID = local_msg->id;
	canREG1->IF1MCTL.bit.DLC = local_msg->dlc;
	// since the data is not kept in an orderly arrangement we'll just do w/o a loop
	canREG1->IF1DATx[3] = local_msg->data[0];
	canREG1->IF1DATx[2] = local_msg->data[1];
	canREG1->IF1DATx[1] = local_msg->data[2];
	canREG1->IF1DATx[0] = local_msg->data[3];
	canREG1->IF1DATx[7] = local_msg->data[4];
	canREG1->IF1DATx[6] = local_msg->data[5];
	canREG1->IF1DATx[5] = local_msg->data[6];
	canREG1->IF1DATx[4] = local_msg->data[7];
	local_msg->state = MSG_STATE_EMPTY;
	canREG1->IF1CMD  = IF_CMD_WR | IF_UPDATE_ARB | IF_UPDATE_DATAA | IF_UPDATE_DATAB |
	    		IF_UPDATE_CONTROL | IF_UPDATE_TXRQST;  // specify which portions we are writing to

	//Todo: is it possible that message ram state changes from above and that is an issue?

	while(CAN1_IF1_BUSY); // wait until interface is not busy
	canREG1->IF1NO = num; // do the copying
	while(CAN1_IF1_BUSY); // wait until interface is done copying

	return SUCCESS;
}

// TODO: how could IF3 be used?
// - how could DMA be used?  Ha! No DMA.
// - should our local_RAM be an "exact" copy of msg_ram?  Decided to only use a portion.
// - any reason to optimize our use of IF1 and IF2?
// - how does FIFO affect all this? Configuration has to set last message in buffer properly.
//      Then it's just a matter of copying all the messages as usual.  Calling code will have to figure out how to read from array of messages.
// - can we use unions to read and write individual parts of IFx registers?
